#!/usr/bin/env python

"""
HTML linking scheme.

Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from moinformat.links.common import Linker
from urllib import quote as _quote
from urlparse import urlparse

def quote(s):

    "Quote URL path 's', preserving path separators and fragment indicators."

    return "#".join(map(_quote, s.split("#", 1)))

class HTMLLinker(Linker):

    "Translate Moin links into HTML links."

    name = "html"

    def get_top_level(self):

        "Return a relative link to the top level."

        levels = self.pagename.count("/")
        return "/".join([".."] * levels)

    def is_url(self, target):

        "Return whether the 'target' references a URL."

        scheme, host, path, params, query, fragment = urlparse(target)
        return scheme and target or None

    def normalise(self, path):

        "Return a normalised form of 'path'."

        return not path.endswith("/") and "%s/" % path or path

    def translate(self, target):

        """
        Translate the 'target', returning a tuple containing the rewritten
        target string and a suitable default label.
        """

        target = target.rstrip("/")

        # Sub-pages.

        if target.startswith("/"):
            return self.translate_subpage(target), None

        # Sibling (of ancestor) pages.

        if target.startswith("../"):
            return self.translate_relative(target), None

        # Attachment or interwiki link.

        rewritten = self.translate_qualified_link(target)
        if rewritten:
            return rewritten # includes label

        # Plain URL.

        rewritten = self.is_url(target)
        if rewritten:
            return rewritten, None

        # Top-level pages.

        top_level = self.get_top_level()
        return quote("%s%s" % (top_level and "%s/" % top_level or "", target)), None

    def translate_qualified_link(self, target):

        """
        Translate a possible qualified link 'target', returning a tuple
        containing a rewritten target and a suitable default label.

        Return None if the link is not suitable.
        """

        t = target.split(":", 1)
        if len(t) != 2:
            return None

        prefix, target = t

        # Attachment links.

        if prefix == "attachment":
            return self.translate_attachment(target), target

        # Interwiki links.

        url = self.mapping.get(prefix)
        if url:
            return self.translate_interwiki(url, target), target

        return None

    # Specific link translators.

    def translate_attachment(self, target):

        "Return a translation of the given attachment 'target'."

        return quote("%sattachments/%s/%s" % (
            self.get_top_level(), self.pagename, target))

    def translate_interwiki(self, url, target):

        "Return a translation of the given interwiki 'target'."

        return "%s%s" % (self.normalise(url), quote(target))

    def translate_relative(self, target):

        "Return a translation of the given relative 'target'."

        return quote(target[len("../"):])

    def translate_subpage(self, target):

        "Return a translation of the given subpage 'target'."

        return quote(".%s" % target)

linker = HTMLLinker

# vim: tabstop=4 expandtab shiftwidth=4
