#!/usr/bin/env python

"""
Directory context functionality.

Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from glob import glob
from os.path import abspath, commonprefix, exists, join

# Get the directory with trailing path separator when assessing path prefixes
# in order to prevent sibling directory confusion.

def inside(filename, dirname):

    "Return whether 'filename' is inside 'dirname'."

    dirname = join(dirname, "")
    return commonprefix((filename, dirname)) == dirname

def within(filename, dirname):

    "Return the part of 'filename' found within 'dirname'."

    dirname = join(dirname, "")
    prefix = commonprefix((filename, dirname))

    if prefix == dirname:
        return filename[len(prefix):]
    else:
        return None



class Directory:

    "A directory abstraction."

    def __init__(self, filename):

        "Initialise the abstraction with the given 'filename'."

        self.filename = abspath(filename)

    def exists(self, filename):

        """
        Return whether 'filename' exists within the directory. This filename
        is relative to the directory.
        """

        return exists(self.get_filename(filename))

    def get_filename(self, filename):

        """
        Return the full path of a file with the given 'filename' found within
        the directory. The full path is an absolute path.
        """

        # Get the absolute path for the combination of directory and filename.

        pathname = abspath(join(self.filename, filename))

        if inside(pathname, self.filename):
            return pathname
        else:
            raise ValueError, filename

    def select_files(self, pattern):

        """
        Return a list of filenames found within the directory matching
        'pattern'. These filenames are relative to the directory.
        """

        full_pattern = self.get_filename(pattern)

        filenames = []

        for filename in glob(full_pattern):
            filename = within(filename, self.filename)
            if filename:
                filenames.append(filename)

        return filenames

# vim: tabstop=4 expandtab shiftwidth=4
